*! version 0.3.2 2014-08-04 | scott long | number
*! version 0.3.1 2014-08-03 | scott long |
*  version 0.3.0 2014-08-02 | scott long | tempnames, new features
*  version 0.2.0 2014-08-01 | scott long |
capture program drop _matrcmath
capture program drop _decodeoptions

program _matrcmath

    version 11.2

    syntax , ismatcol(string) matsource(string) ///
        [ indexout(string) list ///
          isnameopt(string) nameonly(string)   name(string) ///
          isbinary(string)  isunary(string)    isstat(string) ///
          operator(string)  indexleft(string)  indexright(string) ///
          isselect(string)  selectlist(string) ]

//  get matrix and make everything row based

    tempname matcopy vecleft vecright matadd vecnew vectemp vecselect

    matrix `matcopy' = `matsource' // save source till end

if "$debug"=="1" matlist `matcopy', title(matcopy is `matcopy')

    * all work is row operations
    *   : if matcol, transpose matrix
    if `ismatcol' matrix `matcopy' = `matcopy'' // transpose
    local rowsN = rowsof(`matcopy')
    local colsN = colsof(`matcopy')

//  row col selection

    if `isselect' matrix `vecselect' = (`selectlist')
    else matrix `vecselect' = (.) // will select all
    mata: `vecselect' = st_matrix("`vecselect'")

if ("$debug"=="1") matlist `vecselect', title(vecselect)

//  augment matrix

    if `indexout'>`rowsN' {
        local newrowsN = `indexout' - `rowsN'
        matrix `matadd' = J(`newrowsN',`colsN',.z)
        matrix `matcopy' = `matcopy' \ `matadd'
    }
    if `indexout'==0 {
        matrix `matadd' = J(1,`colsN',.z)
        matrix `matcopy' = `matadd' \ `matcopy'
        * since new row, increase all indices
        local indexleft = `indexleft' + 1
        local indexright = `indexright' + 1
        local indexout = `indexout' + 1
    }

if "$debug"=="1" matlist `matcopy', title(after augmentation)

//  binary

    if `isbinary' {

        matrix `vecleft' = `matcopy'[`indexleft',1...]
        matrix `vecright' = `matcopy'[`indexright',1...]
        mata: `vecleft' = st_matrix("`vecleft'")
        mata: `vecright' = st_matrix("`vecright'")

        if ("`operator'"=="-") ///
            mata: `vecnew' = `vecleft' :- `vecright'
        else if ("`operator'"=="+") ///
            mata: `vecnew' = `vecleft' :+ `vecright'
        else if ("`operator'"=="*") ///
            mata: `vecnew' = `vecleft' :* `vecright'
        else if ("`operator'"=="/") ///
            mata: `vecnew' = `vecleft' :/ `vecright'
        else if ("`operator'"=="^") ///
            mata: `vecnew' = `vecleft' :^ `vecright'
        else if ("`operator'"=="%of") ///
            mata: `vecnew' = 100*(`vecleft' :/ `vecright')
        else if ("`operator'"=="%dif") ///
            mata: `vecnew' = 100*((`vecright' :- `vecleft') :/ `vecleft')

        mata: st_matrix("`vecnew'",`vecnew')

    } // binary

//  unary

    if `isunary' {

        matrix `vecleft' = `matcopy'[`indexleft',1...]
        mata: `vecleft' = st_matrix("`vecleft'")

        if ("`operator'"=="exp") ///
            mata: `vecnew' = exp(`vecleft')
        else if ("`operator'"=="log") ///
            mata: `vecnew' = log(`vecleft')
        else if ("`operator'"=="sqrt") ///
            mata: `vecnew' = sqrt(`vecleft')
        else if ("`operator'"=="sq") ///
            mata: `vecnew' = `vecleft' :* `vecleft'
        else if ("`operator'"=="logit") ///
            mata: `vecnew' = log( `vecleft' :/ (1 :- `vecleft') )
        else if ("`operator'"=="odds") ///
            mata: `vecnew' = ( `vecleft' :/ (1 :- `vecleft') )
        else if ("`operator'"=="inv") ///
            mata: `vecnew' = 1 :/  `vecleft'

        mata: st_matrix("`vecnew'",`vecnew')

    } // unary

//  stats and utility

    if `isstat' {

        mata: `matcopy' = st_matrix("`matcopy'")
        local nomata = 0

        if ("`operator'"=="num" | "`operator'"=="number") {
            local nomata = 1
            local matN = colsof(`matcopy')
            numlist "1/`matN'"
            local matnum "`r(numlist)'"
            local matnum : subinstr local matnum " " " , ", all
            matrix `vecnew' = ( `matnum' )
            matrix rowname `vecnew' = "Number"
        }
        else if ("`operator'"=="sum") ///
            mata: `vecnew' = colsum(`matcopy'[`vecselect',.])
        else if ("`operator'"=="mean") ///
            mata: `vecnew' = mean(`matcopy'[`vecselect',.])
        else if ("`operator'"=="var") {
            mata: `vecnew' ///
                = ( colsum(`matcopy'[`vecselect',.] ///
                    :* `matcopy'[`vecselect',.] ) ///
                :- ((colsum(`matcopy'[`vecselect',.]) ///
                :* colsum(`matcopy'[`vecselect',.])) ///
                / rows(`matcopy'[`vecselect',.])) ) ///
                / (rows(`matcopy'[`vecselect',.])-1)
        }
        else if ("`operator'"=="sd") {
            mata: `vecnew' ///
                = sqrt( ///
                ( colsum(`matcopy'[`vecselect',.] ///
                    :* `matcopy'[`vecselect',.] ) ///
                :- ((colsum(`matcopy'[`vecselect',.]) ///
                :* colsum(`matcopy'[`vecselect',.])) ///
                / rows(`matcopy'[`vecselect',.])) ) ///
                / (rows(`matcopy'[`vecselect',.])-1) ///
                )
        }
        else if ("`operator'"=="min") ///
            mata: `vecnew' = colmin(`matcopy'[`vecselect',.])
        else if ("`operator'"=="max") ///
            mata: `vecnew' = colmax(`matcopy'[`vecselect',.])
        else if ("`operator'"=="range")
            mata: `vecnew' = colmax(`matcopy'[`vecselect',.]) ///
                :- colmin(`matcopy'[`vecselect',.])

        if `nomata'!=1 mata: st_matrix("`vecnew'",`vecnew')

    } // stat

//  names

    if (`nameonly'==1) matrix `vecnew' = `matcopy'[`indexout',1...]
    if `isnameopt' matrix rowname `vecnew' = `name'

//  create output matrix

if "$debug"=="1" matlist `vecnew', title(computed vector)

matlist `vecnew'
    capture matrix drop `matsource'
    local rowsN = rowsof(`matcopy')
    forvalues irow = 1/`rowsN' {
        if (`irow'==`indexout') ///
            matrix `matsource' = nullmat(`matsource') \ `vecnew'
        else {
            matrix `vectemp' = `matcopy'[`irow',1...]
            matrix `matsource' = nullmat(`matsource') \ `vectemp'
        }
    }

*if "$debug"=="1" matlist `matsource', title(`matsource')

    * transpose as needed
    if `ismatcol' matrix `matsource' = `matsource''

    if ("`list'"=="list") matlist `matsource', title(`matsource')

end
exit

matrix vecnull = `matcopy', J(`rowsN',1,.z)
* grab names from source matrix that can include spaces
local tmp = colsof(vecnull)
* keeps names, not content
matrix vecnull = vecnull[1...,`tmp']
matrix colnames vecnull = _
* matrix rownames vecnull = `rownms'
local newvecnm "`LEFTnum'_`operator'_`RIGHTnum'"
if `isunary' local newvecnm "`operator'_`RIGHTnum'"
