*! version 0.4.0 2014-08-02 | scott long | select for all operations
*  version 0.3.0 2014-08-02 | scott long | tempname
*  version 0.2.1 2014-08-02 | scott long |
*  version 0.2.0 2014-08-01 | scott long |
capture program drop _matrcsyntax
capture program drop _decodeoptions

program _decodeoptions, sclass
    syntax [ , Name(string) Rows(numlist) Columns(numlist) list * ]
    sreturn local name `name'
    sreturn local rowlist `rows'
    sreturn local collist `columns'
    sreturn local list `list'
end

program _matrcsyntax

    version 11.2

    local binaryops "+ - / * ^ %of %dif"
    local unaryops "sqrt sq log log10 exp inv power odds logit"
    local statops "sum min max mean sd var"

//  row or col

    local rc "`1'" // is it matrow or matcol?
    if ("`rc'"=="row") local ismatcol = 0
    else local ismatcol = 1
    macro shift 1

//  matrix and index for output

    local matsource "`1'" // source matrix
    capture matrix list `matsource'
    if _rc==0 { // matrix exists
        local rowsN = rowsof(`matsource')
        local colsN = colsof(`matsource')
    }
    else {
        display as error "matrix `matsource' does not exist"
        exit
    }
    macro shift 1

//  indexout

    local indexout "`1'"
    if "`indexout'"=="l" { // last
        if `ismatcol' local indexout = `colsN'
        else local indexout = `rowsN'
    }
    else if "`indexout'"=="n" { // next after last
        if `ismatcol' local indexout = `colsN' + 1
        else local indexout = `rowsN' + 1
    }
    else if "`indexout'"=="p" { // prior in front of existing
        local indexout = 0
    }
    else {
        capture confirm integer number `indexout'
        if (_rc==0) local indexout = `1'
        else {
            display as error "index number `indexout' is invalid"
            exit
        }
    }
    macro shift 1

//  decode options

    local input "`*'"
    local newinput : subinstr local input "," " ", all
    _decodeoptions , `newinput'

    local isnameopt = 0
    local name `"`s(name)'"'
    if ("`name'"!="") local isnameopt = 1

    local iscolopt = 0
    local collist "`s(collist)'"
    if ("`collist'"!="") local iscolopt = 1
    local isrowopt = 0
    local rowlist "`s(rowlist)'"
    if ("`rowlist'"!="") local isrowopt = 1

    local list "`s(list)'"

    local nameonly = 0
    if "`1'"!="=" { // if = not simply renaming
        if `isnameopt'==0 {
            display as error "equal sign is missing"
            exit
        }
        else {
            local nameonly = 1
            local isbinary = 0
            local isunary = 0
            local isstat = 0
        }
    }
    macro shift 1

//  decode math

    if !`nameonly' {

        local isbinary = 0
        local isunary = 0
        local isstat = 0
        local iserror = 0

        * is it a binary operator
        local indexleft "`1'" // see if index # is given
        capture confirm integer number `indexleft'
        if _rc==0 {
            local isbinary = 1
            local indexright "`3'"
            confirm integer number `indexright'
            if _rc {
                di as err "index number is required after the operator"
                local iserror = 1
            }
            * check operator
            local operator "`2'"
            local ipos : list posof "`operator'" in binaryops
            if `ipos'==0 {
                display as error "invalid operator `operator'"
                local iserror
            }
            if `iserror' exit
        } // index number so is binary

        * is it a stat operator
        else {
            local indexleft ""
            local operator "`1'"
            local ipos : list posof "`operator'" in statops
            if (`ipos'>0) local isstat = 1
        }

        * is it a unary operator
        if `isbinary'==0 & `isstat'==0 {
            local input "`*'"
            local input : subinstr local input "(" " ", all
            local input : subinstr local input ")" " ", all
            local operator : word 1 of `input'
            local indexleft : word 2 of `input'
            local ipos : list posof "`operator'" in unaryops
            if `ipos'==0 {
                display as error "invalid operator `operator'"
                local iserror
            }
            else local isunary = 1
            capture confirm integer number `indexU'
            if _rc {
                di as err "index number incorrect"
                local iserror = 1
            }
            if `iserror' exit
        }

    } // not nameonly

//  create selection vector

    if `ismatcol' {
        local selectlist "`collist'"
        local isselect = `iscolopt'
    }
    else {
        local selectlist "`rowlist'"
        local isselect = `isrowopt'
    }
    if `isselect' {
        local selectlist = subinstr("`selectlist'"," "," \ ",.)
    }

if "$debug"=="1" {
    di "matsource   `matsource'"
    di "  indexout  `indexout'"
    di "  ismatrow  `ismatrow'  ismatcol  `ismatcol'"
    di
    di "isnameopt   `isnameopt' nameonly  `nameonly'"
    di "  name      `name'      list      `list'"
    di
    di "operator    `operator'  isbinary    `isbinary'"
    di "isunary     `isunary'   isstat      `isstat'"
    di "indexleft   `indexleft' indexright  `indexright'"
    di
    di "isselect    `isselect'  selectlist: `selectlist'"
}

    _matrcmath, ismatcol(`ismatcol') ///
        matsource(`matsource') indexout(`indexout') ///
        isnameopt(`isnameopt') nameonly(`nameonly') name(`name') ///
        operator(`operator') ///
        isunary(`isunary') isbinary(`isbinary') isstat(`isstat') ///
        indexleft(`indexleft') indexright(`indexright') ///
        isselect(`isselect') selectlist(`selectlist') ///
        `list'

end
exit

